@@ -1,26 +1,27 @@
 import argparse
 import fnmatch
 import shutil
-import sys
 import threading
 import traceback
 from copy import deepcopy
 from http.server import HTTPServer
 from pathlib import Path
 
-from kart.utils import KartMap, KartObserver, KartRequesHandler, merge_dicts
+from kart.utils import KartMap, KartObserver, KartRequestHandler, merge_dicts
 
 
 class Kart:
+    """Main Kart class"""
+
     def __init__(
         self,
-        miners=[],
-        content_modifiers=[],
-        mappers=[],
-        map_modifiers=[],
-        renderers=[],
-        config={},
-        build_location="_site",
+        miners: list = [],
+        content_modifiers: list = [],
+        mappers: list = [],
+        map_modifiers: list = [],
+        renderers: list = [],
+        config: list = {},
+        build_location: str = "_site",
     ):
         self.miners = miners
         self.content_modifiers = content_modifiers
@@ -32,15 +33,17 @@ class Kart:
         self.lock = threading.Lock()
 
     def check_config(self):
+        """Checks if the config has all the necessary fields and sets them to default values if not"""
         default = {
             "name": "Example",
-            "base_url": "https://example.org",
+            "site_url": "https://example.org",
             "pagination": {"per_page": 5, "skip": 0},
             "timezone": "UTC",
         }
         merge_dicts(self.config, default)
 
-    def mine_data(self, start=True):
+    def mine_data(self, start: bool = True):
+        """Calls miners and content modifiers"""
         self.site = {"config": self.config}
         for miner in self.miners:
             if start:
@@ -50,17 +53,20 @@ class Kart:
             modifier.modify(self.site)
 
     def create_map(self):
-        self.map = KartMap(self.config["base_url"])
+        """Calls mappers and map modifiers"""
+        self.map = KartMap(site_url=self.config["site_url"])
         for mapper in self.mappers:
             self.map.update(mapper.map(self.site))
         for modifier in self.map_modifiers:
             modifier.modify(self.map, self.site)
 
     def write(self):
+        """Calls renderers"""
         for renderer in self.renderers:
             renderer.render(self.map, self.site, self.build_location)
 
     def build(self):
+        """Build the entire site"""
         self.check_config()
         self.mine_data()
         self.create_map()
@@ -74,7 +80,7 @@ class Kart:
     # preventing errors when serving the site during development
 
     def update_data(self):
-        self.check_config()
+        """Update the site data after a file has been changed"""
         self.mine_data(False)
         self.create_map()
         _site = deepcopy(self.site)
@@ -91,7 +97,8 @@ class Kart:
             self._urls = _urls
             self._regexes = _regexes
 
-    def serve_page(self, handler, url):
+    def serve_page(self, handler, url: str):
+        """Serve a single page"""
         with self.lock:
             site_map = self._map
             urls = self._urls
@@ -108,20 +115,20 @@ class Kart:
         if page:
             self.renderer_dict[page["renderer"]].serve(handler, page, site_map, site)
 
-    def serve(self, port=9000):
+    def serve(self, port: int = 9000):
+        """Main loop for serving the site"""
+        self.check_config()
         self.renderer_dict = {}
-        observer = KartObserver()
-        observer.action = self.update_data
+        observer = KartObserver(action=self.update_data)
         for miner in self.miners:
             miner.start_watching(observer)
         observer.start()
         for renderer in self.renderers:
             self.renderer_dict[renderer.name] = renderer
             renderer.start_serving()
-        handler_class = KartRequesHandler
+        handler_class = KartRequestHandler
         handler_class.action = self.serve_page
         httpd = HTTPServer(("", port), handler_class)
-        httpd.timeout = 0.1
         self.update_data()
         shutil.rmtree(self.build_location, ignore_errors=True)
 
@@ -129,18 +136,20 @@ class Kart:
             try:
                 httpd.handle_request()
             except KeyboardInterrupt:
-                print("\rexiting")
-                for miner in self.miners:
-                    miner.stop_watching()
-                for renderer in self.renderers:
-                    renderer.stop_serving()
-                observer.stop()
-                observer.join()
-                sys.exit()
+                break
             except Exception:
                 print(traceback.format_exc())
 
+        print("\rexiting")
+        for miner in self.miners:
+            miner.stop_watching()
+        for renderer in self.renderers:
+            renderer.stop_serving()
+        observer.stop()
+        observer.join()
+
     def run(self):
+        """Starts the kart execution. See --help for more information"""
         parser = argparse.ArgumentParser()
         parser.add_argument(
             "command", help="command to execute", choices={"build", "serve"}
@@ -148,9 +157,17 @@ class Kart:
         parser.add_argument(
             "-p", "--port", help="port to bind to", default=9000, type=int
         )
+        parser.add_argument(
+            "--dev-url",
+            help="serve your site on a different url",
+            type=str,
+        )
         args = parser.parse_args()
         if args.command == "build":
             self.build()
         if args.command == "serve":
-            self.config["base_url"] = f"http://localhost:{args.port}"
+            if args.dev_url:
+                self.config["site_url"] = args.dev_url
+            else:
+                self.config["site_url"] = f"http://localhost:{args.port}"
             self.serve(args.port)
