@@ -1,4 +1,4 @@
-import shutil
+from abc import ABC, abstractmethod
 from datetime import datetime, time
 from distutils.dir_util import copy_tree
 from http.server import SimpleHTTPRequestHandler
@@ -9,29 +9,40 @@ from dateutil import tz
 from jinja2 import Environment, FileSystemLoader
 
 from kart.markdown import markdown_to_html, markdown_to_toc
-from kart.utils import date_to_string
+from kart.utils import KartDict, KartMap, date_to_string
 
 
-class Renderer:
-    def render(self, map, site, build_location):
-        raise NotImplementedError
+class Renderer(ABC):
+    """Base Renderer class"""
+
+    @abstractmethod
+    def render(self, map: KartMap, site: KartDict, build_location: str):
+        """Renders each page indented for this renderer"""
 
     def start_serving(self):
-        pass
+        """Start the dev server, if necessary"""
 
-    def serve(self, http_handler, page, map, site):
-        raise NotImplementedError
+    @abstractmethod
+    def serve(self, http_handler, page: dict, map: KartMap, site: KartDict):
+        """Serves the the requested page"""
 
     def stop_serving(self):
-        pass
+        """Stops the dev server, if necessary"""
 
 
-class DefaultRenderer(Renderer):
-    def render_single(self, map, site):
-        raise NotImplementedError
+class DefaultFileRenderer(Renderer):
+    """Base class for renderers that render file individually"""
+
+    @abstractmethod
+    def __init__(self, name: str):
+        """Initializes renderer. Must set the ``name`` and ``content_type`` variables"""
+
+    @abstractmethod
+    def render_single(self, page: dict, map: KartMap, site: KartDict) -> str:
+        """Renders a single file"""
 
     def render(self, map, site, build_location):
-        for page in map.values():
+        for page in map:
             if page["renderer"] != self.name:
                 continue
             rendered_file = self.render_single(page, map, site)
@@ -40,24 +51,46 @@ class DefaultRenderer(Renderer):
             with path.open("w") as f:
                 f.write(rendered_file)
 
-    def serve(self, http_handler, page, map, site):
+    def serve(self, http_handler, page: dict, map: KartMap, site: KartDict):
         http_handler.send_response(200)
         http_handler.send_header("Content-type", self.content_type)
         http_handler.end_headers()
         http_handler.wfile.write(bytes(self.render_single(page, map, site), "utf-8"))
 
 
-class DefaultSiteRenderer(DefaultRenderer):
+class DefaultDirectoryRenderer(Renderer):
+    """Base class for renderers that render directories"""
+
+    @abstractmethod
+    def __init__(self, name: str, directory: str):
+        """Initializes renderer. Must set the ``name``, ``dir`` and ``base_url`` variables"""
+
+    def render(self, map: KartMap, site: KartDict, build_location: str):
+        """Copies the entire directory to the target destination"""
+        for page in map:
+            if page["renderer"] != self.name:
+                continue
+            destination = Path(build_location) / Path(*Path(page["url"][:-1]).parts[1:])
+            copy_tree(self.dir, str(destination))
+
+    def serve(self, http_handler, page: dict, map: KartMap, site: KartDict):
+        http_handler.path = self.base_url + http_handler.path
+        return SimpleHTTPRequestHandler.do_GET(http_handler)
+
+
+class DefaultSiteRenderer(DefaultFileRenderer):
+    """Default renderer for rendering html files"""
+
     def __init__(
         self,
-        name="default_site_renderer",
-        template_folder="templates",
-        filters={
+        name: str = "default_site_renderer",
+        template_folder: str = "templates",
+        filters: dict = {
             "html": markdown_to_html,
             "toc": markdown_to_toc,
             "date_to_string": date_to_string,
         },
-        process_count=1,
+        process_count: int = 1,
     ):
         self.name = name
         self.content_type = "text/html"
@@ -66,16 +99,15 @@ class DefaultSiteRenderer(DefaultRenderer):
         self.env = Environment(loader=FileSystemLoader(self.template_folder))
         self.env.filters.update(filters)
 
-    def render_single(self, page, map, site):
-        if self.name != page["renderer"]:
-            return
-
+    def render_single(self, page: dict, map: KartMap, site: KartDict) -> str:
         template = self.env.get_template(page["template"])
         page = {**page["data"], "url": page["url"]}
-
         return template.render(page=page, site=site, url=map.url)
 
-    def _render(self, key, map, site, build_location):
+    def _render_single(
+        self, key: str, map: KartMap, site: KartDict, build_location: str
+    ):
+        """Renderers a single file and saves it to the disk"""
         page = map[key]
         if page["renderer"] != self.name:
             return
@@ -86,26 +118,30 @@ class DefaultSiteRenderer(DefaultRenderer):
         with path.open("w") as f:
             f.write(rendered_file)
 
-    def render(self, map, site, build_location="_site"):
+    def render(self, map: KartMap, site: KartDict, build_location: str = "_site"):
+        """Renderers all the files with a multiprocessing Pool for faster build times"""
         if self.process_count == 1:
-            for key in map:
-                self._render(key, map, site, build_location)
+            for key in map.keys():
+                self._render_single(key, map, site, build_location)
         else:
             with Pool(self.process_count) as p:
-                p.starmap(
-                    self._render, ((key, map, site, build_location) for key in map)
-                )
+                data = ((key, map, site, build_location) for key in map)
+                p.starmap(self._render_single, data)
 
 
-class DefaultFeedRenderer(DefaultRenderer):
-    def __init__(self, name="default_feed_renderer"):
+class DefaultFeedRenderer(DefaultFileRenderer):
+    """Renders an atom feed file"""
+
+    def __init__(self, name: str = "default_feed_renderer"):
+        """Initializes renderer. Sets the ``name`` and ``content_type`` variables"""
         self.name = name
         self.content_type = "application/xml"
 
-    def render_single(self, page, map, site):
+    def render_single(self, page: dict, map: KartMap, site: KartDict) -> str:
+        """Creates the atom feeds"""
         feed_entries = []
         for collection in page["data"]["collections"]:
-            for object in site[collection].values():
+            for object in site[collection]:
                 feed_entries.append([map.url(collection, object["slug"]), object])
         feed_entries.sort(key=lambda x: x[1]["date"], reverse=True)
         atom = '<feed xmlns="http://www.w3.org/2005/Atom">'
@@ -132,51 +168,45 @@ class DefaultFeedRenderer(DefaultRenderer):
         return atom
 
 
-class DefaultSitemapRenderer(DefaultRenderer):
-    def __init__(self, name="default_sitemap_renderer"):
+class DefaultSitemapRenderer(DefaultFileRenderer):
+    """Renders the sitemap of the site"""
+
+    def __init__(self, name: str = "default_sitemap_renderer"):
+        """Initializes renderer. Sets the ``name`` and ``content_type`` variables"""
         self.name = name
         self.content_type = "application/xml"
 
-    def render_single(self, page, map, site):
-        base_url = site["config"]["base_url"]
+    def render_single(self, page: dict, map: KartMap, site: KartDict):
+        """Uses the site ``map`` variable to create the sitemap"""
         sitemap = '<?xml version="1.0" encoding="UTF-8"?>'
         sitemap += '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'
-        for x in map.values():
+        for x in map:
             if x["renderer"] != "default_site_renderer":
                 continue
-            sitemap += f"<url><loc>{base_url+x['url']}</loc></url>"
+            sitemap += f"<url><loc>{map.url(x['url'])}</loc></url>"
         sitemap += "</urlset>"
         return sitemap
 
 
-class DefaultStaticFilesRenderer(Renderer):
-    def __init__(self, name="default_static_files_renderer", directory="static"):
+class DefaultStaticFilesRenderer(DefaultDirectoryRenderer):
+    """Renders all files in the ``static`` directory"""
+
+    def __init__(
+        self, name: str = "default_static_files_renderer", directory: str = "static"
+    ):
+        """Initializes renderer. Sets the ``name``, ``dir`` and ``content_type`` variables"""
         self.name = name
         self.dir = directory
-
-    def render(self, map, site, build_location):
-        for page in map.values():
-            if page["renderer"] != self.name:
-                continue
-            destination = Path(build_location) / Path(*Path(page["url"][:-1]).parts[1:])
-            shutil.copytree(self.dir, destination)
-
-    def serve(self, http_handler, page, map, site):
-        return SimpleHTTPRequestHandler.do_GET(http_handler)
+        self.base_url = ""
 
 
-class DefaultRootDirRenderer(Renderer):
-    def __init__(self, name="default_root_dir_renderer", directory="root"):
+class DefaultRootDirRenderer(DefaultDirectoryRenderer):
+    """Renders all files in the ``root`` directory"""
+
+    def __init__(
+        self, name: str = "default_root_dir_renderer", directory: str = "root"
+    ):
+        """Initializes renderer. Sets the ``name``, ``dir`` and ``content_type`` variables"""
         self.name = name
         self.dir = directory
-
-    def render(self, map, site, build_location):
-        for page in map.values():
-            if page["renderer"] != self.name:
-                continue
-            destination = Path(build_location) / Path(*Path(page["url"][:-1]).parts[1:])
-            copy_tree(self.dir, str(destination))
-
-    def serve(self, http_handler, page, map, site):
-        http_handler.path = "/root" + http_handler.path
-        return SimpleHTTPRequestHandler.do_GET(http_handler)
+        self.base_url = "/root"
